import sys
import os
import json

TOOLS_FOLDER_NAME = 'tools'
INPUT_PROCESSOR_FOLDER_NAME = os.path.join(TOOLS_FOLDER_NAME, "InputProcessor")
INPUT_FOLDER_NAME = os.path.join(INPUT_PROCESSOR_FOLDER_NAME, "Input")
GIVEN_INPUT_FOLDER_NAME = 'Given_Input'

SENG_PROGRAM_URL = 'https://www.uvic.ca/calendar/undergrad/index.php#/programs/SJKVp7AME?bc=true&bcCurrent=Softw'
HISTORICAL_DATA_INPUT_PATH = os.path.join( sys.path[0], INPUT_FOLDER_NAME, GIVEN_INPUT_FOLDER_NAME,'banner.json')
CORE_COURSES_CSV_PATH = os.path.join(sys.path[0], INPUT_FOLDER_NAME, GIVEN_INPUT_FOLDER_NAME, 'core_courses.csv')
CORE_COURSES_INPUT_PATH = os.path.join(sys.path[0], INPUT_FOLDER_NAME,'core_courses.json')
MODEL_1_INPUT_PATH = os.path.join(sys.path[0], INPUT_FOLDER_NAME, "model_1_input.json")
MODEL_2_INPUT_PATH = os.path.join(sys.path[0], INPUT_FOLDER_NAME, "model_2_input.json")


COMPRESSED_HISTORICAL_DATA_PATH =  os.path.join( sys.path[0], INPUT_FOLDER_NAME, GIVEN_INPUT_FOLDER_NAME,'banner.json.bz2')
HISTORICAL_DATA_OUTPUT_PATH =  os.path.join( sys.path[0], INPUT_FOLDER_NAME, GIVEN_INPUT_FOLDER_NAME,'banner.json')

BASE_FEATURES_CSV_PATH = os.path.join(sys.path[0], INPUT_FOLDER_NAME, GIVEN_INPUT_FOLDER_NAME,'base_features.csv')
BASE_FEATURES_INPUT_PATH = os.path.join(sys.path[0], INPUT_FOLDER_NAME, 'base_features.json')

FINAL_INPUT_FEATURES_CSV_PATH = os.path.join(sys.path[0], INPUT_FOLDER_NAME, GIVEN_INPUT_FOLDER_NAME,'final_model_input_features.csv')
FINAL_INPUT_FEATURES_INPUT_PATH = os.path.join(sys.path[0], INPUT_FOLDER_NAME, 'final_model_input_features.json')

FINAL_OUTPUT_FEATURES_CSV_PATH = os.path.join(sys.path[0], INPUT_FOLDER_NAME, GIVEN_INPUT_FOLDER_NAME,'final_model_output_features.csv')
FINAL_OUTPUT_FEATURES_INPUT_PATH = os.path.join(sys.path[0], INPUT_FOLDER_NAME, 'final_model_output_features.json')

CORE_COURSES_OUTPUT_PATH = CORE_COURSES_INPUT_PATH
BASE_FEATURES_OUTPUT_PATH = BASE_FEATURES_INPUT_PATH
FINAL_INPUT_FEATURES_OUTPUT_PATH = FINAL_INPUT_FEATURES_INPUT_PATH
FINAL_OUTPUT_FEATURES_OUTPUT_PATH = FINAL_OUTPUT_FEATURES_INPUT_PATH

ALL_CORE_COURSES_OUTPUT_PATH = os.path.join(sys.path[0], INPUT_FOLDER_NAME, 'all_core_courses.json')
CLEAN_CORE_COURSE_OUTPUT_PATH = os.path.join(sys.path[0], INPUT_FOLDER_NAME, 'cleaned_core_courses.json')
#get data from json file
#returns: list of dictionaries or 1 dictionary

def get_json(input_file_name):
    '''
    This method creates a dictionary containing the
      contents of a json file
    Input:
        - input_file_name: the json file being read from
    Output:
        -  None
    '''
    print("Reading from json file: " + str(input_file_name) + " ..")
    with open(input_file_name, encoding="utf8") as json_file:
        json_data = json.load(json_file)
    print("Data Loaded Succesfully!")
    #print(len(json_data))
    #makes sure historical data does not have enteries that are missing information
    if input_file_name == HISTORICAL_DATA_INPUT_PATH:
        clean_json = [x for x in json_data if x is not None]
        #print(len(clean_json))
        #print(type(json_data))
        return clean_json
    #print(type(json_data))
    return json_data

def create_json(data, output_file_name):
    '''
    This method creates a json file containing the
      contents of a python dictionary
    Input:
        - data: the dictionary which will be written
        - ouput_file_name: the json file being written to
    Output:
        -  None
    '''
    print("Saving to json file: " + str(output_file_name) + " ..")
    with open(output_file_name, 'w', encoding='utf-8') as json_f:
        json_f.write(json.dumps(data, indent=4))



import csv
import tools.InputProcessor.ConfigThings as ConfigThings
import bz2


def create_core_json(csv_input_path, json_output_path):
    '''
    This method creates a json file containing all the SENG core courses
    Input:
        - csv_input_path: csv file containing the names of all the SENG core courses
    Output: 
        - json_output_path: json file containing the names of all the SENG core courses
    '''
    data = {}
    total_courses = 0
    print("Reading from csv file: " + str(csv_input_path) + " ..")
    with open(csv_input_path, 'r', encoding='utf-8-sig') as csv_f:
        csv_data = csv.DictReader(csv_f)
        for rows in csv_data:
            key = rows['course_name']
            data[key] = rows
            total_courses = total_courses + 1
    print("Total Core Courses Found: " + str(total_courses))
    ConfigThings.create_json(data, json_output_path)

def create_feature_json(csv_input_path, json_output_path):
    '''
    This method creates a json file containing all 
      the features of a course that will be tracked
    Input:
        - csv_input_path: csv file containing the names of all 
          the features of a course that will be tracked
    Output: 
        - json_output_path: json file containing the names of all 
          the features of a course that will be tracked
    '''
    data = []
    total_features = 0
    print("Reading from csv file: " + str(csv_input_path) + " ..")
    with open(csv_input_path, 'r', encoding='utf-8-sig') as csv_f:
        csv_data = csv.reader(csv_f)
        for row in csv_data:
            for cell in row:
                if cell != '':
                    data.append(cell)
                    total_features = total_features + 1
    print("Total Features Found: " + str(total_features))
    ConfigThings.create_json(data,json_output_path)
            
def decompress_file(compressed_input_file_path, output_file_path):
    '''This function decompresses a bz file'''
    with open(output_file_path, 'wb') as new_file, bz2.BZ2File(compressed_input_file_path, 'rb') as file:
        decompressor = bz2.BZ2Decompressor()
        for data in iter(lambda: file.read(100*1024), b''):
            new_file.write(data)


if __name__ == '__main__':
    create_core_json(ConfigThings.CORE_COURSES_CSV_PATH, ConfigThings.CORE_COURSES_OUTPUT_PATH)
    create_feature_json(ConfigThings.BASE_FEATURES_CSV_PATH, ConfigThings.BASE_FEATURES_OUTPUT_PATH)
    decompress_file(ConfigThings.COMPRESSED_HISTORICAL_DATA_PATH, ConfigThings.HISTORICAL_DATA_OUTPUT_PATH)


import numpy as np
from sklearn import semi_supervised
from sklearn.linear_model import LinearRegression
from tools.InputProcessor import ConfigThings
from tools.InputProcessor import DataProcessor

# !OA - Do we really want to use tools in our main workflow? Can we duplicate
#       this functionality into src so we can have tools as a top-level folder
#       (along with src and test).

def create_model_inputs():
    """creates DataProcessor Object which initializes model input data
    """
    data_processor = DataProcessor.DataProcessor()
    complete_historical_core_courses = data_processor.get_historical_clean_core_course_offerings()
    model_1_input, model_2_input = data_processor.get_historical_core_courses(complete_historical_core_courses)
    data_processor.print_model_1_input_to_json(model_1_input)
    data_processor.print_model_2_input_to_json(model_2_input)
    return model_1_input, model_2_input, data_processor.core_course_names

def process_model_1_input(model_1_input):
    """Creates Model_1_input (ALl Courses) Object which initializes formats model 1 input data
    """
    x_years = {}
    y_enrollments = {}
    for course in model_1_input:
        x_years[course], y_enrollments[course] = create_model_1_input(model_1_input[course])
    return x_years ,y_enrollments

def create_model_1_input(course):
    """Creates Model_1_input (1 course) 
    """
    x_year = [] # x_year[i] = course[year].key()[ij] (year i, semester j)
    y_enrollment = [] # y_enrollment[i] = course[year][semester]["maximumEnrollment"]
    for i_year in course:
        for semester in course[i_year].values():
            if semester:
                x_year.append(int(i_year))
                y_enrollment.append(int(semester["maximumEnrollment"]))
    x_year = np.array(x_year, dtype=np.int32).reshape(-1,1)
    y_enrollment = np.array(y_enrollment, dtype=np.int32)
    return x_year, y_enrollment
    
def round_floats_to_ints(float_list):
    int_list = []
    for float_value in float_list:
        int_value = round(float_value)
        int_list.append(int_value)
    return int_list

def main(verbose=False):
    model_1_inputs, model_2_inputs, core_course_names =  create_model_inputs()
    x_years, y_enrollments = process_model_1_input(model_1_inputs)
    model = LinearRegression()
    for course in core_course_names:
        model.fit(x_years[course], y_enrollments[course])
        r_sq = model.score(x_years[course],y_enrollments[course])
        y_pred_floats = model.predict(x_years[course])
        y_pred = round_floats_to_ints(y_pred_floats)
        if verbose:
            print("    **** "+ str(course) + " Predicted Sizes ****")
            for input, pred in zip(x_years[course],y_pred):
                print("Year: " + str(int(input)) + "    Prediction:" + str(pred))
    # !OA/!JS - We should return a dict of output here.
    return {"Real": "Data"}

if __name__ == '__main__':
    main()


from anyio import SemaphoreStatistics
import tools.InputProcessor.Helper as Helper
from tools.InputProcessor.ConfigThings import *
from functools import partial, total_ordering
import os
import sys

# We should not be printing verbosely if this is called from an external component (such as the main workflow).
is_main = (__name__ == "__main__")
if not is_main:
    f = open(os.devnull, 'w')
    sys.stdout = f

class DataProcessor():
    """ In charge of Processing Data for Predictor
    """
    def __init__(self):
        self.setup_input_files()
        self.setup_input_dictionaries()
        self.core_course_names = self.create_core_course_names_list(self.core_courses_dict)
        self.core_course_indicies_dict = self.create_core_course_indicies_dict(self.core_course_names)
    
    def setup_input_files(self):
        """ Creates all files necessary for the model's input data
            to be extracted
        """
        #create json file containing core courses from .csv file
        Helper.create_core_json(CORE_COURSES_CSV_PATH, CORE_COURSES_OUTPUT_PATH)
        #create json file containing base features from .csv file
        Helper.create_feature_json(BASE_FEATURES_CSV_PATH, BASE_FEATURES_OUTPUT_PATH)
        #create json file containing final input features for predictive model from .csv file
        Helper.create_feature_json(FINAL_INPUT_FEATURES_CSV_PATH, FINAL_INPUT_FEATURES_OUTPUT_PATH)
        #create json file containing final output features for predictive model from .csv file
        Helper.create_feature_json(FINAL_OUTPUT_FEATURES_CSV_PATH, FINAL_OUTPUT_FEATURES_OUTPUT_PATH)
        #decompress json file containing historical offerings from .bz file
        Helper.decompress_file(COMPRESSED_HISTORICAL_DATA_PATH, HISTORICAL_DATA_OUTPUT_PATH)

    def setup_input_dictionaries(self):
        """" Creates all dictionaries necessary for the model's input
             data to be extracted
        """
        #create list containing names of core SENG courses from .json file
        self.core_courses_dict = get_json(CORE_COURSES_INPUT_PATH)
        #create list containing names of base features from .json file
        self.base_features = get_json(BASE_FEATURES_INPUT_PATH)
        #create list containing names of final input features of prediction model
        self.final_input_features = get_json(FINAL_INPUT_FEATURES_INPUT_PATH)
        #create list containing names of final output features of prediction model
        self.final_output_features = get_json(FINAL_OUTPUT_FEATURES_OUTPUT_PATH)

    def create_core_course_names_list(self, core_courses_dict):
        '''
            This method creates a list of the names of all core_courses
            using a dictionary containing all the core courses
            Input:
                - core_courses_dict: dictionary containing all core courses and 
                    their attributes
            Output:
                - core_courses: list of core course names
        '''
        core_courses = [x for x in core_courses_dict if x is not None]
        return core_courses
    
    def create_core_course_indicies_dict(self, core_courses):
        index = 1
        core_course_indicies_dict = {}
        for core_course in core_courses:
            core_course_indicies_dict[core_course] = index
            index = index + 1
        return core_course_indicies_dict

    def get_historical_clean_core_course_offerings(self):
        """ Retrieves all historical offferings
        """
        historical_offerings = get_json(HISTORICAL_DATA_INPUT_PATH)
        f1_1 = lambda offering: self.remove_useless_offering_features(offering, self.base_features)
        clean_historical_core_course_offerings = self.extract_historical_core_offerings_list(f1_1, historical_offerings, self.core_courses_dict)
        complete_historical_core_course_offerings = self.add_offering_features(clean_historical_core_course_offerings, self.core_courses_dict )
        return complete_historical_core_course_offerings
    
    #calls functions 2
    def get_historical_core_courses(self, historical_core_course_offeirngs):
        """ Retrieves all historical courses
        """
        historical_core_courses = self.extract_all_historical_core_courses(historical_core_course_offeirngs, self.core_course_names)
        f2_1 = lambda core_course_years : self.remove_model_1_unwanted_features(core_course_years)
        f3_1 = lambda core_course_years, semester: self.group_all_courses_by_semester_helper(core_course_years, semester, self.final_input_features)
        yearly_course_offerings, model_1_input = self.group_all_courses_by_year(f2_1, historical_core_courses, self.core_course_names)
        model_2_input = self.group_all_courses_by_semester(f3_1, yearly_course_offerings)
        return model_1_input, model_2_input
    

    #functions 2
    def core_course_offering_check(self, offering, core_course_names):
        '''
        This method checks if a course is a core course
        Input: 
            - offering: a course offering (dictionary)
            - core_course_namess: list of names of core SENG courses
        Output:
            - True: if offering belongs to a core course
            - False: otherwise
    '''
        for row in core_course_names:
            if offering["subjectCourse"] in row:
                return True
        return False

    def add_offering_features(self, historical_core_course_offerings, core_courses):
        complete_offerings = []
        for offering in historical_core_course_offerings:
            offering["term_year"] = offering["term"][0:4]
            offering["semester"] = offering["term"][4:6]
            del offering["term"]
            offering["course_year"] = core_courses[offering["subjectCourse"]]["course_year"]
            offering["course_faculty"] =  core_courses[offering["subjectCourse"]]["course_faculty"]
            offering["course_substitutions"] =  core_courses[offering["subjectCourse"]]["course_substitutions"]
            complete_offerings.append(offering)
        return complete_offerings

    def extract_historical_core_offerings_list(self, f1_1, historical_offerings, core_courses_dict ):
        """
    This method extracts  all core course offerings found in all historical
     offerings
    Input:
        - historical_core_offerings: List of offerings (dictionaries) offered historically
        - core_courses_dict: dictionary containing core SENG courses and their information
    Output:
        - historical_core_offerings: list of dictionaries containing offerings in core SENG
            courses offered historically
    """
        print("Retreiving All Core Courses Offered Historically..")
        historical_core_course_offerings = []
        core_course_names = self.create_core_course_names_list(core_courses_dict)
        for offering in historical_offerings:
            #skip all offerings that are not a lecture
            if not self.offering_is_lecture(offering):
                continue
            if self.core_course_offering_check(offering, core_course_names):
                offering = f1_1(offering)
                historical_core_course_offerings.append(offering)
        print("Total Core Course Offerings: " + str(len(historical_core_course_offerings)) + " out of " + str(len(historical_offerings)) + " Offerings")    
        return historical_core_course_offerings

    def extract_historical_core_course(self, historical_core_offerings, core_course_name):
        '''
        This method extracts one core course's historical offerings from all
        from all historical core SENG course offerings
        Input:
            - historical_core_offerings: List of all core SENG courses' historical
                offerings
            - core_course_name: name of a core SENG course
        '''
        core_course_offers = []
        for hist_core_offering in historical_core_offerings:
            if str(hist_core_offering["subjectCourse"]) == str(core_course_name):
                core_course_offers.append(hist_core_offering)
        return core_course_offers

    def extract_all_historical_core_courses(self, historical_core_offerings, core_course_names):
        '''
        This method extracts a dictionary of core SENG courses, where each course is
            represented by a list of offerings. Each offerings is a dictionary
        Input:
            - historical_core_offerings: List of all core SENG course historical
                offerings
            - core_course_names: List of the names of all core SENG courses
        Output:
            - historical_core_courses: Dictionary of all core SENG courses' 
                historical offerings
        '''
        historical_core_course_offerings = []
        historical_core_courses = {}
        for core_course_name in core_course_names:
            historical_core_course_offerings = self.extract_historical_core_course(historical_core_offerings, core_course_name)
            historical_core_courses[historical_core_course_offerings[0]["subjectCourse"]] = historical_core_course_offerings
        return historical_core_courses

    def group_all_courses_by_year(self, f2_1, historical_core_courses, core_courses_names):
        '''
        This function groups all core SENG course offerings by course name and
        the year the course was offered
        Input:
            - historical_core_courses: Dictionary of all core SENG courses' 
                historical offerings
            - core_courses_dict: dictionary containing core SENG courses and their information
        Output:
            - yearly_core_courses: Dictionary containing all offerings for a core SENG course,
                each core SENG course is further organized by the year an offer was made'''
        print("Grouping Offerings for each Course by Year")
        yearly_core_courses = {}
        model_1_input = {}
        for core_course in core_courses_names:
            yearly_course_offerings = {} #dictionary of all offerings for a course by  year
            model_1_course = {}
            for year in range(2008,2023):
                year_offerings_list = [] # list of all offerings for a course in a year
                model_1_year = {}
                for offering in historical_core_courses[core_course]:
                    if offering['term_year'] == str(year):
                        year_offerings_list.append(offering)
                yearly_course_offerings[year] = year_offerings_list
                model_1_year[year] = f2_1(yearly_course_offerings[year])
                model_1_course[year] = model_1_year
            model_1_input[core_course] = model_1_course
            yearly_core_courses[core_course] = yearly_course_offerings
        return yearly_core_courses, model_1_input

    def group_all_courses_by_semester(self, f3_1, historical_core_courses_yearly):
        print("Grouping Offerings for each year of a course by semester..")
        SEMESTER_VALUES = ["Fall", "Spring", "Summer"]
        model_2_input = {}
        for core_course in self.core_course_names:
            if historical_core_courses_yearly[core_course]:
                model_2_course = {}
                for i_year in range(2008,2023):
                    if historical_core_courses_yearly[core_course][i_year]:
                        model_2_year = {}
                        for semester in SEMESTER_VALUES:
                            model_2_semester = {}
                            model_2_semester = f3_1(historical_core_courses_yearly[core_course][i_year], semester)
                            if model_2_semester:
                                model_2_year[semester] = model_2_semester
                        model_2_course[i_year] = model_2_year
                model_2_input[core_course] = model_2_course
        return model_2_input                    

#helper functions
    def find_total_offerings_for_course(self, course):
        total_offerings = 0
        for offering in course:
            total_offerings = self.add_to_total_offerings(total_offerings)
        return total_offerings

    def add_to_total_offerings(self, total_offerings):
        return total_offerings + 1
    
    def add_to_total_offerings_size(self, offering_size, total_enrollment):
        return total_enrollment + offering_size

    def add_to_total_seats(self, seats, total_seats):
        return total_seats + seats

    def offering_is_lecture(self, offering):
        if offering["scheduleTypeDescription"] == "Lecture":
            return True
        else:
            return False

    def rename_semester_feature(self, offering):
        if offering["semester"] == "01":
            offering["semester"] = "Spring"
        if offering["semester"] == "05":
            offering["semester"] = "Summer"
        if offering["semester"]  == "09":
            offering["semester"] = "Fall"
        return offering

    def create_model_1_input(self, core_course_year):
        total_seats = 0
        total_offerings_size = 0
        total_offerings = 0
        for offering in core_course_year:
            total_seats = self.add_to_total_seats(offering["enrollment"], total_seats)
            total_offerings_size = self.add_to_total_offerings_size(offering["maximumEnrollment"], total_offerings_size)
            total_offerings = self.add_to_total_offerings(total_offerings)
        return total_seats, total_offerings_size, total_offerings

#lambda functions
    #f1_1
    def remove_useless_offering_features(self, offering, base_features):
        clean_offering = {feature: offering[feature] for feature in base_features}
        return clean_offering
    #f2_1
    def group_all_courses_by_year_helper(self, core_course_dicts, year,final_input_features):
        """
        """
        for offering in core_course_dicts:
            offering = self.rename_semester_feature(offering)

    #f2_1
    def remove_model_1_unwanted_features(self, core_course_year):
        new_core_course_semester = {}
        total_seats = 0
        total_offerings_size = 0
        total_offerings = 0
        total_seats, total_offerings_size, total_offerings = self.create_model_1_input(core_course_year)
        if total_seats != 0:
            new_core_course_semester["maximumEnrollment"] = total_offerings_size
            new_core_course_semester["totalEnrollment"] = total_seats
        return new_core_course_semester
        
    #f3_1
    def group_all_courses_by_semester_helper(self, core_course_years, semester, final_input_features):
        semester_offerings_list = []
        for offering in core_course_years:
            offering = self.rename_semester_feature(offering)
            if offering["semester"] == semester:
                clean_offering = {feature: offering[feature] for feature in final_input_features}
                semester_offerings_list.append(clean_offering)

        return semester_offerings_list

    def print_model_1_input_to_json(self, model_input):
        create_json(model_input, MODEL_1_INPUT_PATH)
    
    def print_model_2_input_to_json(self, model_output):
        create_json(model_output, MODEL_2_INPUT_PATH)


import sys
import Predictor

# !RP/!JC - Let's load and return a pickle? 
def dummyData() -> dict:
    return {"Dummy": "Data"}

def main() -> dict:
    try:
        if sys.argv[1] == "dummy":
            return dummyData()
    except IndexError:
        pass
    return Predictor.main()

if __name__ == "__main__":
    main()


# !RP/!JC - Fill in these methods. We made need to adjust the parameters/output
#           format.

import json
import string
from tools.InputProcessor import DataProcessor

def parse_input_for_sizer(backend_input_data) -> dict:
    # Dictionary, where keys are courses e.g. "CSC111" and values are dicts.
    # Each dict within a course contains the following structure:
    # {
    #   "1": 10,
    #   "2": 10,
    #   "2T": 10,
    #   "3": 10,
    #   "4": 10,
    #   "5": 10,
    #   "6": 10,
    #   "7": 10,
    #   "Enrolment": 50,
    #   "Maximum Enrolment": 60,
    #   "Year": 2008
    pass

# ENUM: 0, 1, 2 for Fall Spring Summer

def parse_input_for_sequencer(input_data) -> dict:
    # Outermost: Dictionary, where keys are courses e.g. "CSC111" and values are dicts.
    # Each dict within a course contains the following structure:
    # {
    #   "1": 10,
    #   "2": 10,
    #   "2T": 10,
    #   "3": 10,
    #   "4": 10,
    #   "5": 10,
    #   "6": 10,
    #   "7": 10,
    #   "Fall Enrolment": 50,
    #   "Fall Maximum Enrolment": 60,
    #   "Spring Enrolment": 50,
    #   "Spring Maximum Enrolment": 60,
    #   "Summer Enrolment": 50,
    #   "Summer Maximum Enrolment": 60,
    #   "Year Enrolment": 50,
    #   "Year Maximum Enrolment": 60,
    #   "Year": 2008
    pass

def parse_output_for_backend(predictor_output: dict, prev_year: int, dummy: bool) -> dict:
    # predictor_output:
    #
    # Outermost: Dictionary, where keys are courses e.g. "CSC111" and values are lists of numpy arrays.
    # Each list contains 3 numpy arrays with values(capacities for all sections): Fall Maximum Enrolment, Spring Maximum Enrolment, Summer Maximum Enrolment.
    # Example: {"CSC111": [[150], [100], [80]], "CSC115": [[250], [200], [150]], "MATH100": [[250, 250, 250], [80, 80], [70]]....}
    #
    # Return Value:
    # Refer to Data Model.
    
    fall_offerings = []
    spring_offerings = []
    summer_offerings = []

    if dummy: return(build_dummy_schedule())

    first_year_copy = get_prev_year_course_data(prev_year)
    predictor_output.update(first_year_copy)

    for key in predictor_output:
        value = predictor_output[key]
        
        # if value[0] is non-empty then the course sections contained there should be scheduled in the FALL semester.
        if value[0] != []: 
            fall_offerings.append(build_offering_dict(key, value[0]))
        # if value[1] is non-empty then the course sections contained at there should be scheduled in the SPRING semester.
        if value[1] != []:
            spring_offerings.append(build_offering_dict(key, value[1]))
        # if value[2] is non-empty then the course sections contained at there should be scheduled in the SUMMER semester.
        if value[2] != []:
            summer_offerings.append(build_offering_dict(key, value[2]))

    return {"fall": fall_offerings,
            "spring": spring_offerings,
            "summer": summer_offerings}

def build_offering_dict(course_code: string, sections: list) -> dict:
    offering_dict = {
        "course": {
            "code": course_code,
            #TODO: !RP can we add the course title to the core courses csv?
            "title": "",
            "pengRequired": False
        },
        "sections": []
    }
    for capacity in sections:
        offering_dict['sections'].append({
            "professor": None,
            "capacity": capacity,
            "timeSlots": None
        })
    
    return(offering_dict)

#TODO: !RP Don't know if we still need this? passing dummy data to parse_output_for_backend() instead of a 'dummy' flag will do the same thing.
def build_dummy_schedule(prev_first_year_data: dict) -> dict:
    print("you are a dummy!")
    return({})

def get_prev_year_course_data(current_year):
    data_processor = DataProcessor.DataProcessor()
    complete_core_courses = data_processor.get_historical_clean_core_course_offerings()

    combined_dict = {}
    #TODO: get timeslots for these first year courses/sections.

    for course in complete_core_courses:
        if(course['courseNumber'][0] == '1' and course['subject'] != 'CSC' and course['subject'] != 'SENG'):
            if course['subjectCourse'] in combined_dict:
                if(course['term_year'] == str(current_year) and course['semester'] == '09'): # fall
                    combined_dict[course['subjectCourse']][0].append(course['maximumEnrollment'])
                if(course['term_year'] == str(current_year) and course['semester'] == '01'): # spring
                    combined_dict[course['subjectCourse']][1].append(course['maximumEnrollment'])
                if(course['term_year'] == str(current_year) and course['semester'] == '05'): # summer
                    combined_dict[course['subjectCourse']][2].append(course['maximumEnrollment'])
            else:
                if(course['term_year'] == str(current_year) and course['semester'] == '09'): # fall
                    combined_dict[course['subjectCourse']] = [[course['maximumEnrollment']],[],[]]
                if(course['term_year'] == str(current_year) and course['semester'] == '01'): # spring
                    combined_dict[course['subjectCourse']] = [[],[course['maximumEnrollment']],[]]
                if(course['term_year'] == str(current_year) and course['semester'] == '05'): # summer
                    combined_dict[course['subjectCourse']] = [[],[],[course['maximumEnrollment']]]

    # out_file1 = open("combined.json", "w")
    # json.dump(combined_dict, out_file1, indent = 4)

    return(combined_dict)

# def main():
#     result = parse_output_for_backend({"CSC111": [[100], [150], []], "CSC115": [[100], [150], []], "SENG370": [[100], [150], [20]], "ECE260": [[100], [150], [60]]}, 2022, False)
#     out_file = open("dummy_out.json", "w")
#     json.dump(result, out_file, indent=6)

#     if result != None:
#         print(result)

# if __name__ == '__main__':
#     main()


import sys
import numpy as np 
import matplotlib.pyplot as plt
import json

def Graph(filePath, shifted, displayAllYears):

    # Opening JSON file
    with open(filePath) as json_file:
        data = json.load(json_file)
    
    fileName = data[0]['subjectCourse']

    offering = {}
    Spring = []
    Summer = []
    Fall = []
    SpringMax = []
    SummerMax = []
    FallMax = []

    year = ['2007', '2008', '2009', '2010', 
            '2011', '2012', '2013', 
            '2014', '2015', '2016', 
            '2017', '2018', '2019', 
            '2020', '2021', '2022']

    # Display all years
    for yr in year:
        offering[yr] = {'summer': [0,0], 'fall': [0,0], 'spring': [0,0]}

    for term in data:
        # Spring
        if term['term'][-2:] == '01':
            if shifted:
                # adjusting spring offering back to correct year
                adjustedYear = str(int(term['term'][0:4]) - 1)
                offering[adjustedYear]['spring'][0] += term['enrollment']
                offering[adjustedYear]['spring'][1] += (term['maximumEnrollment'] - term['enrollment'])
            else:
                offering[term['term'][0:4]]['spring'][0] += term['enrollment']
                offering[term['term'][0:4]]['spring'][1] += (term['maximumEnrollment'] - term['enrollment'])

        # Summer
        if term['term'][-2:] == '05':
            if shifted:
                # adjusting spring offering back to correct year
                adjustedYear = str(int(term['term'][0:4]) - 1)
                offering[adjustedYear]['summer'][0] += term['enrollment']
                offering[adjustedYear]['summer'][1] += (term['maximumEnrollment'] - term['enrollment'])
            else:
                offering[term['term'][0:4]]['summer'][0] += term['enrollment']
                offering[term['term'][0:4]]['summer'][1] += (term['maximumEnrollment'] - term['enrollment'])

        # Fall
        if term['term'][-2:] == '09':
            offering[term['term'][0:4]]['fall'][0] += term['enrollment']
            offering[term['term'][0:4]]['fall'][1] += (term['maximumEnrollment'] - term['enrollment'])
    
    for item in offering:
        if displayAllYears:
            Summer.append(offering[item]['summer'][0])
            Fall.append(offering[item]['fall'][0])
            Spring.append(offering[item]['spring'][0])

            SummerMax.append(offering[item]['summer'][1])
            FallMax.append(offering[item]['fall'][1])
            SpringMax.append(offering[item]['spring'][1])
        else:
            if(offering[item]['summer'][0] == 0 and offering[item]['fall'][0] == 0 and offering[item]['spring'][0] == 0):
                year.remove(item)
            else:
                Summer.append(offering[item]['summer'][0])
                Fall.append(offering[item]['fall'][0])
                Spring.append(offering[item]['spring'][0])

                SummerMax.append(offering[item]['summer'][1])
                FallMax.append(offering[item]['fall'][1])
                SpringMax.append(offering[item]['spring'][1])
    

    # Graph stuff
    x_axis = np.arange(len(year))

    # Multi bar Chart
    plt.bar(x_axis +0.20, Summer, width = 0.2, label = 'Summer')
    plt.bar(x_axis +0.20, SummerMax, width = 0.2, bottom = Summer, color = '0.75')

    plt.bar(x_axis +0.20 * 2, Fall, width = 0.2, label = 'Fall')
    plt.bar(x_axis +0.20 * 2, FallMax, width = 0.2, bottom = Fall, color = '0.50')

    plt.bar(x_axis +0.20 * 3, Spring, width = 0.2, label = 'Spring')
    plt.bar(x_axis +0.20 * 3, SpringMax, width = 0.2, bottom = Spring, color = '0.75')


    # Xticks
    plt.xticks(x_axis, year)
    
    # Labels
    plt.xlabel("Year")
    plt.ylabel("# Students Enrolled")
    plt.title(fileName)

    plt.legend()
    plt.plot()
    plt.xticks(rotation = 90)
    plt.savefig('images/' + str(fileName + "_bar"))
    # plt.show()
    plt.clf()
    plt.close()

def main():
    shifted = False
    displayAllYears = False

    for arg in sys.argv:
        if arg == '-s':
            shifted = True
        if arg == '-f':
            displayAllYears = True
    
    print("Graphing " + sys.argv[1])
    Graph(sys.argv[1], shifted, displayAllYears)

if __name__ == "__main__":
    main()


import json
import bz2

#decompress json file containing historical offerings from .bz file
with open('./Input/Given_Input/banner.json', 'wb') as new_file, bz2.BZ2File('./Input/Given_Input/banner.json.bz2', 'rb') as file:
    decompressor = bz2.BZ2Decompressor()
    for data in iter(lambda: file.read(100*1024), b''):
        new_file.write(data)

data_file = open('./Input/Given_Input/banner.json', 'r')
semester_json = json.load(data_file)

filtered = []
filtered_by_course = {}

# CENG 412 maps to ECE 440
# CENG 420 to ECE 470
# CENG 421 to ECE 471
# CENG 450 to ECE 449
# CENG 453 to ECE 457
# CENG 460 to ECE 458
# CENG 461 to ECE 463
# CENG 496?
# ECE 498 is merged CENG 498 and ELEC 498
# ECE 499 is merged CENG 499 and ELEC 499

for datum in semester_json:
    if datum is not None:
        if (datum['subject'] == 'SENG' or datum['subject'] == 'CSC' or datum['subject'] == 'ECE') and datum['scheduleTypeDescription'] == 'Lecture':
            filtered_datum = {
                'term': datum['term'],
                'subject': datum['subject'],
                'courseNumber': datum['courseNumber'],
                'sequenceNumber': datum['sequenceNumber'],
                'scheduleTypeDescription': datum['scheduleTypeDescription'],
                'courseTitle': datum['courseTitle'],
                'maximumEnrollment': datum['maximumEnrollment'],
                'enrollment': datum['enrollment'],
                'seatsAvailable': datum['seatsAvailable'],
                'subjectCourse': datum['subjectCourse']
            }
                
            filtered.append(filtered_datum)

        if (datum['subject'] == 'CENG' or datum['subject'] == 'ELEC') and datum['scheduleTypeDescription'] == 'Lecture':
            if datum['subjectCourse'] == 'CENG412':
                datum['courseNumber'] == '440'
            elif datum['subjectCourse'] == 'CENG420':
                datum['courseNumber'] == '470'
            elif datum['subjectCourse'] == 'CENG421':
                datum['courseNumber'] == '471'
            elif datum['subjectCourse'] == 'CENG450':
                datum['courseNumber'] == '449'
            elif datum['subjectCourse'] == 'CENG453':
                datum['courseNumber'] == '457'
            elif datum['subjectCourse'] == 'CENG460':
                datum['courseNumber'] == '458'
            elif datum['subjectCourse'] == 'CENG461':
                datum['courseNumber'] == '463'

            filtered_datum = {
                'term': datum['term'],
                'subject': 'ECE',
                'courseNumber': datum['courseNumber'],
                'sequenceNumber': datum['sequenceNumber'],
                'scheduleTypeDescription': datum['scheduleTypeDescription'],
                'courseTitle': datum['courseTitle'],
                'maximumEnrollment': datum['maximumEnrollment'],
                'enrollment': datum['enrollment'],
                'seatsAvailable': datum['seatsAvailable'],
                'subjectCourse': 'ECE' + datum['courseNumber']
            }
                
            filtered.append(filtered_datum)

filtered_file = open('./Input/Given_Input/banner_clean.json', 'w')

filtered_file.write(json.dumps(filtered, indent=4))

filtered_file.close()
data_file.close()
